@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-bin.c: Basic container actor
  *
@@ -39,13 +40,15 @@
 #include "mx-bin.h"
 #include "mx-enum-types.h"
 #include "mx-private.h"
-#include "mx-stylable.h"
+#include "mx-focusable.h"
+
 
 #define MX_BIN_GET_PRIVATE(obj)       (G_TYPE_INSTANCE_GET_PRIVATE ((obj), MX_TYPE_BIN, MxBinPrivate))
 
 struct _MxBinPrivate
 {
   ClutterActor *child;
+  gboolean      child_has_space;
 
   MxAlign       x_align;
   MxAlign       y_align;
@@ -66,10 +69,13 @@ enum
 };
 
 static void clutter_container_iface_init (ClutterContainerIface *iface);
+static void mx_bin_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (MxBin, mx_bin, MX_TYPE_WIDGET,
                                   G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
-                                                         clutter_container_iface_init));
+                                                         clutter_container_iface_init)
+                                  G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                         mx_bin_focusable_iface_init));
 
 void
 _mx_bin_get_align_factors (MxBin   *bin,
@@ -160,6 +166,24 @@ clutter_container_iface_init (ClutterContainerIface *iface)
   iface->foreach = mx_bin_foreach;
 }
 
+static MxFocusable*
+mx_bin_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  MxBinPrivate *priv = MX_BIN (focusable)->priv;
+
+  if (MX_IS_FOCUSABLE (priv->child))
+    return mx_focusable_accept_focus (MX_FOCUSABLE (priv->child), hint);
+  else
+    return NULL;
+}
+
+static void
+mx_bin_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->accept_focus = mx_bin_accept_focus;
+}
+
+
 static void
 mx_bin_paint (ClutterActor *self)
 {
@@ -168,8 +192,8 @@ mx_bin_paint (ClutterActor *self)
   /* allow MxWidget to paint the background */
   CLUTTER_ACTOR_CLASS (mx_bin_parent_class)->paint (self);
 
-  /* the pain our child */
-  if (priv->child)
+  /* then paint our child */
+  if (priv->child && priv->child_has_space)
     clutter_actor_paint (priv->child);
 }
 
@@ -223,6 +247,17 @@ mx_bin_allocate_child (MxBin                 *bin,
       if (available_height < 0)
         available_height = 0;
 
+      /* If we have no available space, just bail out. We'll skip
+       * drawing the child.
+       */
+      if (available_width == 0 || available_height == 0)
+        {
+          priv->child_has_space = FALSE;
+          return;
+        }
+      else
+        priv->child_has_space = TRUE;
+
       if (priv->x_fill)
         {
           allocation.x1 = (int) padding.left;
@@ -418,7 +453,7 @@ mx_bin_set_property (GObject      *gobject,
 
     case PROP_Y_FILL:
       mx_bin_set_fill (bin,
-                       bin->priv->y_fill,
+                       bin->priv->x_fill,
                        g_value_get_boolean (value));
       break;
 
@@ -552,6 +587,7 @@ mx_bin_init (MxBin *bin)
 
   bin->priv->x_align = MX_ALIGN_MIDDLE;
   bin->priv->y_align = MX_ALIGN_MIDDLE;
+  bin->priv->child_has_space = TRUE;
 }
 
 /**
@@ -610,7 +646,7 @@ mx_bin_set_child (MxBin        *bin,
  *
  * Retrieves a pointer to the child of @bin.
  *
- * Return value: a #ClutterActor, or %NULL
+ * Return value: (transfer none): a #ClutterActor, or %NULL
  */
 ClutterActor *
 mx_bin_get_child (MxBin *bin)
