@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-button.c: Plain button actor
  *
@@ -40,6 +41,7 @@
 #include <glib.h>
 
 #include <clutter/clutter.h>
+#include <clutter/clutter-keysyms.h>
 
 #include "mx-button.h"
 
@@ -50,14 +52,15 @@
 #include "mx-texture-cache.h"
 #include "mx-private.h"
 #include "mx-enum-types.h"
+#include "mx-focusable.h"
 
 enum
 {
   PROP_0,
 
   PROP_LABEL,
-  PROP_TOGGLE,
-  PROP_ACTIVE
+  PROP_IS_TOGGLE,
+  PROP_TOGGLED
 };
 
 enum
@@ -74,26 +77,28 @@ struct _MxButtonPrivate
 {
   gchar            *text;
 
-  ClutterActor     *old_bg;
-  gboolean          old_bg_parented; /* TRUE if we have adopted old_bg */
-
   guint8            old_opacity;
 
   guint             is_pressed : 1;
   guint             is_hover : 1;
-  guint             is_checked : 1;
   guint             is_toggle : 1;
+  guint             is_toggled : 1;
 
   ClutterAnimation *animation;
+
+  ClutterActor *content_image;
 };
 
 static guint button_signals[LAST_SIGNAL] = { 0, };
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
+static void mx_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxButton, mx_button, MX_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                mx_stylable_iface_init));
+                                                mx_stylable_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_focusable_iface_init));
 
 static void
 mx_stylable_iface_init (MxStylableIface *iface)
@@ -113,9 +118,64 @@ mx_stylable_iface_init (MxStylableIface *iface)
                                         &bg_color,
                                         G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_BUTTON, pspec);
+
+      pspec = g_param_spec_boxed ("x-mx-content-image",
+                                   "Content Image",
+                                   "Image used as the button",
+                                   MX_TYPE_BORDER_IMAGE,
+                                   G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_BUTTON, pspec);
+
+      pspec = g_param_spec_string ("x-mx-icon-name",
+                                   "Icon name",
+                                   "Named icon to place inside the button",
+                                   NULL,
+                                   G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_BUTTON, pspec);
+
+      pspec = g_param_spec_int ("x-mx-icon-size",
+                                "Icon size",
+                                "Size to use for icon",
+                                1, G_MAXINT, 48,
+                                G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_BUTTON, pspec);
     }
 }
 
+static MxFocusable*
+mx_button_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  /* FIXME: pretend hover is the same as focus for now */
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "hover");
+  MX_BUTTON (focusable)->priv->is_hover = TRUE;
+
+  clutter_actor_grab_key_focus (CLUTTER_ACTOR (focusable));
+
+  return focusable;
+}
+
+static MxFocusable*
+mx_button_move_focus (MxFocusable      *focusable,
+                      MxFocusDirection  direction,
+                      MxFocusable      *from)
+{
+  /* check if focus is being moved from us */
+  if (focusable == from)
+    {
+      mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "");
+      MX_BUTTON (focusable)->priv->is_hover = FALSE;
+    }
+
+  return NULL;
+}
+
+static void
+mx_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->accept_focus = mx_button_accept_focus;
+  iface->move_focus = mx_button_move_focus;
+}
+
 static void
 mx_button_update_label_style (MxButton *button)
 {
@@ -127,98 +187,148 @@ mx_button_update_label_style (MxButton *button)
   if (!CLUTTER_IS_TEXT (label))
     return;
 
-  _mx_widget_set_clutter_text_attributes (MX_WIDGET (button),
-                                          CLUTTER_TEXT (label));
+  mx_stylable_apply_clutter_text_attributes (MX_STYLABLE (button),
+                                             CLUTTER_TEXT (label));
 }
 
-static void
-mx_button_dispose_old_bg (MxButton *button)
-{
-  MxButtonPrivate *priv = button->priv;
-
-  if (priv->old_bg)
-    {
-      if (priv->old_bg_parented)
-        {
-          clutter_actor_unparent (priv->old_bg);
-          priv->old_bg_parented = FALSE;
-        }
-      g_object_unref (priv->old_bg);
-      priv->old_bg = NULL;
-    }
-}
-
-static void
-mx_button_stylable_changed (MxStylable *stylable)
-{
-  MxButton *button = MX_BUTTON (stylable);
-  ClutterActor *bg_image;
-
-  mx_button_dispose_old_bg (button);
-
-  bg_image = mx_widget_get_border_image ((MxWidget*) button);
-  if (bg_image)
-    button->priv->old_bg = g_object_ref (bg_image);
-}
-
-static void
-mx_animation_completed (ClutterAnimation *animation,
-                        MxButton         *button)
-{
-  mx_button_dispose_old_bg (button);
-}
 
 static void
 mx_button_style_changed (MxWidget *widget)
 {
   MxButton *button = MX_BUTTON (widget);
   MxButtonPrivate *priv = button->priv;
+  MxBorderImage *content_image = NULL;
+  gchar *icon_name = NULL;
+  gint icon_size = 48;
 
   /* update the label styling */
   mx_button_update_label_style (button);
 
-  /* run a transition if applicable */
-  if (priv->old_bg &&
-      (!mx_stylable_get_style_pseudo_class (MX_STYLABLE (widget))))
-    {
-      ClutterAnimation *animation;
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "x-mx-content-image", &content_image,
+                   "x-mx-icon-name", &icon_name,
+                   "x-mx-icon-size", &icon_size,
+                   NULL);
 
-      if (!clutter_actor_get_parent (priv->old_bg))
+  if (content_image)
+    {
+      GError *err = NULL;
+
+      if (priv->content_image)
         {
-          clutter_actor_set_parent (priv->old_bg, (ClutterActor*) widget);
-          priv->old_bg_parented = TRUE;
+          clutter_actor_unparent (priv->content_image);
         }
 
-      animation = clutter_actor_animate (priv->old_bg,
-                                         CLUTTER_LINEAR,
-                                         120,
-                                         "opacity", 0,
-                                         NULL);
-      g_signal_connect (animation, "completed",
-                        G_CALLBACK (mx_animation_completed), button);
+      priv->content_image = clutter_texture_new_from_file (content_image->uri,
+                                                           &err);
+
+      if (priv->content_image)
+        clutter_actor_set_parent (priv->content_image, CLUTTER_ACTOR (widget));
+
+      if (err)
+        {
+          g_warning ("Could not load content image: %s", err->message);
+
+          g_error_free (err);
+        }
+
+      g_boxed_free (MX_TYPE_BORDER_IMAGE, content_image);
+      g_free (icon_name);
+
+      if (icon_name)
+        g_warning ("The \"-mx-icon-name\" property is ignored when"
+                   " \"-mx-content-image\" is set");
+
+      if (mx_widget_get_background_image (widget))
+        g_warning ("The \"background-image\" property is ignored when"
+                   " \"-mx-content-image\" is set");
+
+      if (mx_widget_get_border_image (widget))
+        g_warning ("The \"border-image\" property is ignored when"
+                   " \"-mx-content-image\" is set");
+
+      return;
     }
+
+  if (icon_name)
+    {
+      ClutterActor *icon = mx_bin_get_child (MX_BIN (widget));
+
+      if (!icon || !MX_IS_ICON (icon))
+        {
+          icon = mx_icon_new ();
+          mx_bin_set_child (MX_BIN (widget), icon);
+        }
+
+      mx_icon_set_icon_size (MX_ICON (icon), icon_size);
+      mx_icon_set_icon_name (MX_ICON (icon), icon_name);
+
+      g_free (icon_name);
+    }
+
+
 }
 
 
+static void
+mx_button_push (MxButton           *button,
+                ClutterButtonEvent *event)
+{
+  MxWidget *widget = MX_WIDGET (button);
+
+  mx_widget_hide_tooltip (widget);
+
+  button->priv->is_pressed = TRUE;
+
+  //clutter_grab_pointer (CLUTTER_ACTOR (button));
+
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "active");
+
+  if (event)
+    mx_widget_long_press_query (widget, event);
+}
+
+static void
+mx_button_pull (MxButton *button)
+{
+  MxWidget *widget = MX_WIDGET (button);
+  MxButtonPrivate *priv = button->priv;
+
+  if (!button->priv->is_pressed)
+    return;
+
+  //clutter_ungrab_pointer ();
+
+  if (button->priv->is_toggle)
+    {
+      mx_button_set_toggled (button, !button->priv->is_toggled);
+    }
+
+  button->priv->is_pressed = FALSE;
+
+  g_signal_emit (button, button_signals[CLICKED], 0);
+
+  mx_widget_long_press_cancel (widget);
+
+  if (priv->is_toggled)
+    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "checked");
+  else if (!priv->is_hover)
+    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), NULL);
+  else
+    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "hover");
+}
 
 static gboolean
 mx_button_button_press (ClutterActor       *actor,
                         ClutterButtonEvent *event)
 {
-  MxButtonPrivate *priv = MX_BUTTON (actor)->priv;
-  MxWidget *widget = MX_WIDGET (actor);
 
-  mx_widget_hide_tooltip (widget);
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return TRUE;
 
   if (event->button == 1)
     {
-      priv->is_pressed = TRUE;
-
-      clutter_grab_pointer (actor);
-
-      mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "active");
-
-      mx_widget_long_press_query (widget, event);
+      mx_button_push (MX_BUTTON (actor), event);
 
       return TRUE;
     }
@@ -230,34 +340,12 @@ static gboolean
 mx_button_button_release (ClutterActor       *actor,
                           ClutterButtonEvent *event)
 {
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return TRUE;
+
   if (event->button == 1)
     {
-      MxButton *button = MX_BUTTON (actor);
-      MxWidget *widget = MX_WIDGET (actor);
-      MxButtonPrivate *priv = button->priv;
-
-      if (!button->priv->is_pressed)
-        return FALSE;
-
-      clutter_ungrab_pointer ();
-
-      if (button->priv->is_toggle)
-        {
-          mx_button_set_checked (button, !button->priv->is_checked);
-        }
-
-      button->priv->is_pressed = FALSE;
-
-      g_signal_emit (button, button_signals[CLICKED], 0);
-
-      mx_widget_long_press_cancel (widget);
-
-      if (priv->is_checked)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "checked");
-      else if (!priv->is_hover)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), NULL);
-      else
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
+      mx_button_pull (MX_BUTTON (actor));
 
       return TRUE;
     }
@@ -265,6 +353,34 @@ mx_button_button_release (ClutterActor       *actor,
   return TRUE;
 }
 
+static gboolean
+mx_button_key_press (ClutterActor    *actor,
+                     ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_Return || event->keyval == CLUTTER_space)
+    {
+      mx_button_push (MX_BUTTON (actor), NULL);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+mx_button_key_release (ClutterActor    *actor,
+                       ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_Return || event->keyval == CLUTTER_space)
+    {
+      mx_button_pull (MX_BUTTON (actor));
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static gboolean
 mx_button_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
@@ -275,14 +391,20 @@ mx_button_enter (ClutterActor         *actor,
   if (event->source != actor)
     return FALSE;
 
-  if (!button->priv->is_checked)
+  /* hide the tooltip */
+  if (mx_widget_get_tooltip_text (widget))
+    mx_widget_show_tooltip (widget);
+
+  /* check if the widget is disabled */
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return FALSE;
+
+
+  if (!button->priv->is_toggled)
     mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
 
   button->priv->is_hover = 1;
 
-  if (mx_widget_get_has_tooltip (widget))
-    mx_widget_show_tooltip (widget);
-
   return FALSE;
 }
 
@@ -296,23 +418,29 @@ mx_button_leave (ClutterActor         *actor,
   if (event->source != actor)
     return FALSE;
 
+  /* hide the tooltip */
+  if (mx_widget_get_tooltip_text (widget))
+    mx_widget_hide_tooltip (widget);
+
+  /* check if the widget is disabled */
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return FALSE;
+
+
   button->priv->is_hover = 0;
 
   if (button->priv->is_pressed)
     {
-      clutter_ungrab_pointer ();
+      //clutter_ungrab_pointer ();
 
       button->priv->is_pressed = FALSE;
     }
 
-  if (button->priv->is_checked)
+  if (button->priv->is_toggled)
     mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "checked");
   else
     mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), NULL);
 
-  if (mx_widget_get_has_tooltip (widget))
-    mx_widget_hide_tooltip (widget);
-
   return FALSE;
 }
 
@@ -329,11 +457,11 @@ mx_button_set_property (GObject      *gobject,
     case PROP_LABEL:
       mx_button_set_label (button, g_value_get_string (value));
       break;
-    case PROP_TOGGLE:
-      mx_button_set_toggle_mode (button, g_value_get_boolean (value));
+    case PROP_IS_TOGGLE:
+      mx_button_set_is_toggle (button, g_value_get_boolean (value));
       break;
-    case PROP_ACTIVE:
-      mx_button_set_checked (button, g_value_get_boolean (value));
+    case PROP_TOGGLED:
+      mx_button_set_toggled (button, g_value_get_boolean (value));
       break;
 
     default:
@@ -355,11 +483,11 @@ mx_button_get_property (GObject    *gobject,
     case PROP_LABEL:
       g_value_set_string (value, priv->text);
       break;
-    case PROP_TOGGLE:
+    case PROP_IS_TOGGLE:
       g_value_set_boolean (value, priv->is_toggle);
       break;
-    case PROP_ACTIVE:
-      g_value_set_boolean (value, priv->is_checked);
+    case PROP_TOGGLED:
+      g_value_set_boolean (value, priv->is_toggled);
       break;
 
     default:
@@ -381,7 +509,13 @@ mx_button_finalize (GObject *gobject)
 static void
 mx_button_dispose (GObject *gobject)
 {
-  mx_button_dispose_old_bg (MX_BUTTON (gobject));
+  MxButtonPrivate *priv = MX_BUTTON (gobject)->priv;
+
+  if (priv->content_image)
+    {
+      clutter_actor_unparent (priv->content_image);
+      priv->content_image = NULL;
+    }
 
   G_OBJECT_CLASS (mx_button_parent_class)->dispose (gobject);
 }
@@ -393,8 +527,8 @@ mx_button_map (ClutterActor *self)
 
   CLUTTER_ACTOR_CLASS (mx_button_parent_class)->map (self);
 
-  if (priv->old_bg && priv->old_bg_parented)
-    clutter_actor_map (priv->old_bg);
+  if (priv->content_image)
+    clutter_actor_map (priv->content_image);
 }
 
 static void
@@ -404,25 +538,29 @@ mx_button_unmap (ClutterActor *self)
 
   CLUTTER_ACTOR_CLASS (mx_button_parent_class)->unmap (self);
 
-  if (priv->old_bg && priv->old_bg_parented)
-    clutter_actor_unmap (priv->old_bg);
+  if (priv->content_image)
+    clutter_actor_unmap (priv->content_image);
 }
 
 static void
 mx_button_paint_background (MxWidget           *widget,
-                           ClutterActor       *background,
-                           const ClutterColor *color)
+                            ClutterActor       *background,
+                            const ClutterColor *color)
 {
   MxButtonPrivate *priv;
 
+  priv = MX_BUTTON (widget)->priv;
+
+  if (priv->content_image)
+    {
+      clutter_actor_paint (priv->content_image);
+
+      return;
+    }
+
   MX_WIDGET_CLASS (mx_button_parent_class)->paint_background (widget,
                                                               background,
                                                               color);
-
-  priv = MX_BUTTON (widget)->priv;
-
-  if (priv->old_bg && priv->old_bg_parented)
-    clutter_actor_paint (priv->old_bg);
 }
 
 static void
@@ -430,11 +568,78 @@ mx_button_allocate (ClutterActor           *actor,
                     const ClutterActorBox  *box,
                     ClutterAllocationFlags  flags)
 {
+  MxButtonPrivate *priv = MX_BUTTON (actor)->priv;
+
   CLUTTER_ACTOR_CLASS (mx_button_parent_class)->allocate (actor, box, flags);
 
+  if (priv->content_image)
+    {
+      ClutterActorBox childbox;
+
+      childbox.x1 = 0;
+      childbox.y1 = 0;
+      childbox.x2 = (box->x2 - box->x1);
+      childbox.y2 = (box->y2 - box->y1);
+
+      clutter_actor_allocate (priv->content_image, &childbox, flags);
+    }
+
   mx_bin_allocate_child (MX_BIN (actor), box, flags);
 }
 
+static void
+mx_button_get_preferred_width (ClutterActor *actor,
+                               gfloat        for_height,
+                               gfloat       *min_width,
+                               gfloat       *pref_width)
+{
+  MxButtonPrivate *priv = MX_BUTTON (actor)->priv;
+
+  if (priv->content_image)
+    {
+      clutter_actor_get_preferred_width (priv->content_image, for_height,
+                                         min_width, pref_width);
+      return;
+    }
+
+  CLUTTER_ACTOR_CLASS (mx_button_parent_class)->get_preferred_width (actor,
+                                                                     for_height,
+                                                                     min_width,
+                                                                     pref_width);
+}
+
+static void
+mx_button_get_preferred_height (ClutterActor *actor,
+                                gfloat        for_width,
+                                gfloat       *min_height,
+                                gfloat       *pref_height)
+{
+  MxButtonPrivate *priv = MX_BUTTON (actor)->priv;
+
+  if (priv->content_image)
+    {
+      clutter_actor_get_preferred_height (priv->content_image, for_width,
+                                          min_height, pref_height);
+      return;
+    }
+
+  CLUTTER_ACTOR_CLASS (mx_button_parent_class)->get_preferred_height (actor,
+                                                                      for_width,
+                                                                      min_height,
+                                                                      pref_height);
+}
+
+static void
+mx_button_paint (ClutterActor *actor)
+{
+  MxButtonPrivate *priv = MX_BUTTON (actor)->priv;
+
+  if (priv->content_image)
+    clutter_actor_paint (priv->content_image);
+  else
+    CLUTTER_ACTOR_CLASS (mx_button_parent_class)->paint (actor);
+}
+
 static void
 mx_button_class_init (MxButtonClass *klass)
 {
@@ -452,8 +657,13 @@ mx_button_class_init (MxButtonClass *klass)
 
   actor_class->button_press_event = mx_button_button_press;
   actor_class->button_release_event = mx_button_button_release;
+  actor_class->key_press_event = mx_button_key_press;
+  actor_class->key_release_event = mx_button_key_release;
   actor_class->enter_event = mx_button_enter;
   actor_class->leave_event = mx_button_leave;
+  actor_class->get_preferred_width = mx_button_get_preferred_width;
+  actor_class->get_preferred_height = mx_button_get_preferred_height;
+  actor_class->paint = mx_button_paint;
 
   actor_class->map = mx_button_map;
   actor_class->unmap = mx_button_unmap;
@@ -464,21 +674,21 @@ mx_button_class_init (MxButtonClass *klass)
   pspec = g_param_spec_string ("label",
                                "Label",
                                "Label of the button",
-                               NULL, G_PARAM_READWRITE);
+                               NULL, MX_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_LABEL, pspec);
 
-  pspec = g_param_spec_boolean ("toggle-mode",
-                                "Toggle Mode",
+  pspec = g_param_spec_boolean ("is-toggle",
+                                "Is Toggle",
                                 "Enable or disable toggling",
-                                FALSE, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TOGGLE, pspec);
+                                FALSE, MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_IS_TOGGLE, pspec);
 
-  pspec = g_param_spec_boolean ("checked",
-                                "Checked",
+  pspec = g_param_spec_boolean ("toggled",
+                                "Toggled",
                                 "Indicates if a toggle button is \"on\""
                                 " or \"off\"",
-                                FALSE, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_ACTIVE, pspec);
+                                FALSE, MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_TOGGLED, pspec);
 
   /**
    * MxButton::clicked:
@@ -507,9 +717,6 @@ mx_button_init (MxButton *button)
 
   g_signal_connect (button, "style-changed",
                     G_CALLBACK (mx_button_style_changed), NULL);
-
-  g_signal_connect (button, "stylable-changed",
-                    G_CALLBACK (mx_button_stylable_changed), NULL);
 }
 
 /**
@@ -596,13 +803,11 @@ mx_button_set_label (MxButton    *button,
       mx_bin_set_child ((MxBin*) button, label);
     }
 
-  mx_stylable_changed ((MxStylable*) button);
-
   g_object_notify (G_OBJECT (button), "label");
 }
 
 /**
- * mx_button_get_toggle_mode:
+ * mx_button_get_is_toggle:
  * @button: a #MxButton
  *
  * Get the toggle mode status of the button.
@@ -610,7 +815,7 @@ mx_button_set_label (MxButton    *button,
  * Returns: #TRUE if toggle mode is set, otherwise #FALSE
  */
 gboolean
-mx_button_get_toggle_mode (MxButton *button)
+mx_button_get_is_toggle (MxButton *button)
 {
   g_return_val_if_fail (MX_IS_BUTTON (button), FALSE);
 
@@ -618,7 +823,7 @@ mx_button_get_toggle_mode (MxButton *button)
 }
 
 /**
- * mx_button_set_toggle_mode:
+ * mx_button_set_is_toggle:
  * @button: a #Mxbutton
  * @toggle: #TRUE or #FALSE
  *
@@ -626,51 +831,51 @@ mx_button_get_toggle_mode (MxButton *button)
  * state will be "toggled" when the user clicks the button.
  */
 void
-mx_button_set_toggle_mode (MxButton *button,
-                           gboolean  toggle)
+mx_button_set_is_toggle (MxButton *button,
+                         gboolean  toggle)
 {
   g_return_if_fail (MX_IS_BUTTON (button));
 
   button->priv->is_toggle = toggle;
 
-  g_object_notify (G_OBJECT (button), "toggle-mode");
+  g_object_notify (G_OBJECT (button), "is-toggle");
 }
 
 /**
- * mx_button_get_checked:
+ * mx_button_get_toggled:
  * @button: a #MxButton
  *
  * Get the state of the button that is in toggle mode.
  *
- * Returns: #TRUE if the button is checked, or #FALSE if not
+ * Returns: #TRUE if the button is toggled, or #FALSE if not
  */
 gboolean
-mx_button_get_checked (MxButton *button)
+mx_button_get_toggled (MxButton *button)
 {
   g_return_val_if_fail (MX_IS_BUTTON (button), FALSE);
 
-  return button->priv->is_checked;
+  return button->priv->is_toggled;
 }
 
 /**
- * mx_button_set_checked:
+ * mx_button_set_toggled:
  * @button: a #Mxbutton
- * @checked: #TRUE or #FALSE
+ * @toggled: #TRUE or #FALSE
  *
- * Sets the pressed state of the button. This is only really useful if the
+ * Sets the toggled state of the button. This is only really useful if the
  * button has #toggle-mode mode set to #TRUE.
  */
 void
-mx_button_set_checked (MxButton *button,
-                       gboolean  checked)
+mx_button_set_toggled (MxButton *button,
+                       gboolean  toggled)
 {
   g_return_if_fail (MX_IS_BUTTON (button));
 
-  if (button->priv->is_checked != checked)
+  if (button->priv->is_toggled != toggled)
     {
-      button->priv->is_checked = checked;
+      button->priv->is_toggled = toggled;
 
-      if (checked)
+      if (toggled)
         mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "checked");
       else
       if (button->priv->is_hover)
@@ -678,6 +883,6 @@ mx_button_set_checked (MxButton *button,
       else
         mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), NULL);
 
-      g_object_notify (G_OBJECT (button), "checked");
+      g_object_notify (G_OBJECT (button), "toggled");
     }
 }
