@@ -1,8 +1,9 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-scroll-bar.c: Scroll bar actor
  *
  * Copyright 2008 OpenedHand
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -71,6 +72,8 @@ struct _MxScrollBarPrivate
   gfloat        move_x;
   gfloat        move_y;
 
+  guint         handle_min_size;
+
   /* Trough-click handling. */
   enum { NONE, UP, DOWN }  paging_direction;
   guint             paging_source_id;
@@ -79,9 +82,7 @@ struct _MxScrollBarPrivate
   gboolean          stepper_forward;
   guint             stepper_source_id;
 
-  ClutterAnimation *paging_animation;
-
-  gboolean          vertical;
+  MxOrientation     orientation;
 };
 
 enum
@@ -89,7 +90,7 @@ enum
   PROP_0,
 
   PROP_ADJUSTMENT,
-  PROP_VERTICAL
+  PROP_ORIENTATION
 };
 
 enum
@@ -121,8 +122,8 @@ mx_scroll_bar_get_property (GObject    *gobject,
       g_value_set_object (value, priv->adjustment);
       break;
 
-    case PROP_VERTICAL:
-      g_value_set_boolean (value, priv->vertical);
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, priv->orientation);
       break;
 
     default:
@@ -145,27 +146,8 @@ mx_scroll_bar_set_property (GObject      *gobject,
       mx_scroll_bar_set_adjustment (bar, g_value_get_object (value));
       break;
 
-    case PROP_VERTICAL:
-      bar->priv->vertical = g_value_get_boolean (value);
-      if (bar->priv->vertical)
-        {
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->bw_stepper),
-                                  "up-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->fw_stepper),
-                                  "down-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
-                                  "vhandle");
-        }
-      else
-        {
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->fw_stepper),
-                                  "forward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->bw_stepper),
-                                  "backward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
-                                  "hhandle");
-        }
-      clutter_actor_queue_relayout ((ClutterActor*) gobject);
+    case PROP_ORIENTATION:
+      mx_scroll_bar_set_orientation (bar, g_value_get_enum (value));
       break;
 
     default:
@@ -276,6 +258,98 @@ mx_scroll_bar_unmap (ClutterActor *actor)
     clutter_actor_unmap (priv->handle);
 }
 
+static void
+mx_scroll_bar_get_preferred_width (ClutterActor *actor,
+                                   gfloat        for_height,
+                                   gfloat       *min_width,
+                                   gfloat       *pref_width)
+{
+  MxScrollBarPrivate *priv = MX_SCROLL_BAR (actor)->priv;
+  MxPadding padding;
+  gfloat width;
+
+  mx_widget_get_padding (MX_WIDGET (actor), &padding);
+
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
+    {
+      gfloat bg_w, h_w, bs_w, fs_w;
+
+      clutter_actor_get_preferred_width (priv->bw_stepper, -1, NULL, &bs_w);
+      clutter_actor_get_preferred_width (priv->fw_stepper, -1, NULL, &fs_w);
+      clutter_actor_get_preferred_width (priv->trough, -1, NULL, &bg_w);
+      clutter_actor_get_preferred_width (priv->handle, -1, NULL, &h_w);
+
+      h_w += padding.left + padding.right;
+      bs_w += padding.left + padding.right;
+      fs_w += padding.left + padding.right;
+
+      width = MAX (bg_w, MAX (h_w, MAX (bs_w, fs_w)));
+    }
+  else
+    {
+      gfloat fs_w, bs_w;
+
+      clutter_actor_get_preferred_width (priv->bw_stepper, -1, NULL, &bs_w);
+      clutter_actor_get_preferred_width (priv->fw_stepper, -1, NULL, &fs_w);
+
+      width = padding.left + fs_w + priv->handle_min_size + bs_w
+        + padding.right;
+
+    }
+
+  if (min_width)
+    *min_width = width;
+
+  if (pref_width)
+    *pref_width = width;
+}
+
+static void
+mx_scroll_bar_get_preferred_height (ClutterActor *actor,
+                                    gfloat        for_width,
+                                    gfloat       *min_height,
+                                    gfloat       *pref_height)
+{
+  MxScrollBarPrivate *priv = MX_SCROLL_BAR (actor)->priv;
+  MxPadding padding;
+  gfloat height;
+
+  mx_widget_get_padding (MX_WIDGET (actor), &padding);
+
+  if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
+    {
+      gfloat bg_h, h_h, bs_h, fs_h;
+
+      clutter_actor_get_preferred_height (priv->bw_stepper, -1, NULL, &bs_h);
+      clutter_actor_get_preferred_height (priv->fw_stepper, -1, NULL, &fs_h);
+      clutter_actor_get_preferred_height (priv->trough, -1, NULL, &bg_h);
+      clutter_actor_get_preferred_height (priv->handle, -1, NULL, &h_h);
+
+      h_h += padding.top + padding.bottom;
+      bs_h += padding.top + padding.bottom;
+      fs_h += padding.top + padding.bottom;
+
+      height = MAX (bg_h, MAX (h_h, MAX (bs_h, fs_h)));
+    }
+  else
+    {
+      gfloat fs_h, bs_h;
+
+      clutter_actor_get_preferred_height (priv->bw_stepper, -1, NULL, &bs_h);
+      clutter_actor_get_preferred_height (priv->fw_stepper, -1, NULL, &fs_h);
+
+      height = padding.top + fs_h + priv->handle_min_size + bs_h
+        + padding.bottom;
+    }
+
+  if (min_height)
+    *min_height = height;
+
+  if (pref_height)
+    *pref_height = height;
+
+}
+
 static void
 mx_scroll_bar_allocate (ClutterActor          *actor,
                         const ClutterActorBox *box,
@@ -287,8 +361,8 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
   gfloat x, y, width, height, stepper_size;
 
   /* Chain up */
-  CLUTTER_ACTOR_CLASS (mx_scroll_bar_parent_class)->
-  allocate (actor, box, flags);
+  CLUTTER_ACTOR_CLASS (mx_scroll_bar_parent_class)->allocate (actor, box,
+                                                              flags);
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
@@ -298,7 +372,7 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
   width = (box->x2 - box->x1) - padding.left - padding.right;
   height = (box->y2 - box->y1) - padding.top - padding.bottom;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       stepper_size = width;
 
@@ -366,15 +440,18 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
                                 NULL,
                                 &page_size);
 
+      value = mx_adjustment_get_value (priv->adjustment);
+
       if ((upper == lower)
           || (page_size >= (upper - lower)))
         increment = 1.0;
       else
         increment = page_size / (upper - lower);
 
+      min_size = priv->handle_min_size;
+
       mx_stylable_get (MX_STYLABLE (actor),
-                       "min-size", &min_size,
-                       "max-size", &max_size,
+                       "mx-max-size", &max_size,
                        NULL);
 
       if (upper - lower - page_size <= 0)
@@ -382,7 +459,7 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
       else
         position = (value - lower) / (upper - lower - page_size);
 
-      if (priv->vertical)
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
           avail_size = height - stepper_size * 2;
           handle_size = increment * avail_size;
@@ -420,14 +497,18 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
 }
 
 static void
-mx_scroll_bar_style_changed (MxWidget *widget)
+mx_scroll_bar_style_changed (MxWidget *widget, MxStyleChangedFlags flags)
 {
   MxScrollBarPrivate *priv = MX_SCROLL_BAR (widget)->priv;
 
-  mx_stylable_changed ((MxStylable *) priv->bw_stepper);
-  mx_stylable_changed ((MxStylable *) priv->fw_stepper);
-  mx_stylable_changed ((MxStylable *) priv->trough);
-  mx_stylable_changed ((MxStylable *) priv->handle);
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "mx-min-size", &priv->handle_min_size,
+                   NULL);
+
+  mx_stylable_style_changed ((MxStylable *) priv->bw_stepper, flags);
+  mx_stylable_style_changed ((MxStylable *) priv->fw_stepper, flags);
+  mx_stylable_style_changed ((MxStylable *) priv->trough, flags);
+  mx_stylable_style_changed ((MxStylable *) priv->handle, flags);
 
 }
 
@@ -492,14 +573,16 @@ mx_scroll_bar_scroll_event (ClutterActor       *actor,
       if (value == lower)
         return FALSE;
       else
-        mx_adjustment_set_value (priv->adjustment, value - step);
+        mx_adjustment_interpolate_relative (priv->adjustment, -step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     case CLUTTER_SCROLL_DOWN:
     case CLUTTER_SCROLL_RIGHT:
       if (value == upper)
         return FALSE;
       else
-        mx_adjustment_set_value (priv->adjustment, value + step);
+        mx_adjustment_interpolate_relative (priv->adjustment, step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     }
 
@@ -520,6 +603,8 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
   object_class->dispose      = mx_scroll_bar_dispose;
   object_class->constructor  = mx_scroll_bar_constructor;
 
+  actor_class->get_preferred_width  = mx_scroll_bar_get_preferred_width;
+  actor_class->get_preferred_height = mx_scroll_bar_get_preferred_height;
   actor_class->allocate       = mx_scroll_bar_allocate;
   actor_class->paint          = mx_scroll_bar_paint;
   actor_class->pick           = mx_scroll_bar_pick;
@@ -536,12 +621,13 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
                                       MX_TYPE_ADJUSTMENT,
                                       MX_PARAM_READWRITE));
 
-  pspec = g_param_spec_boolean ("vertical",
-                                "Vertical Orientation",
-                                "Vertical Orientation",
-                                FALSE,
-                                MX_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_VERTICAL, pspec);
+  pspec = g_param_spec_enum ("orientation",
+                             "Orientation",
+                             "The orientation of the scrollbar",
+                             MX_TYPE_ORIENTATION,
+                             MX_ORIENTATION_HORIZONTAL,
+                             MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_ORIENTATION, pspec);
 
   signals[SCROLL_START] =
     g_signal_new ("scroll-start",
@@ -573,7 +659,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
 
       is_initialized = TRUE;
 
-      pspec = g_param_spec_uint ("min-size",
+      pspec = g_param_spec_uint ("mx-min-size",
                                  "Minimum grabber size",
                                  "Minimum size of the scroll grabber, in px",
                                  0, G_MAXUINT, 32,
@@ -581,7 +667,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
       mx_stylable_iface_install_property (iface,
                                           MX_TYPE_SCROLL_BAR, pspec);
 
-      pspec = g_param_spec_uint ("max-size",
+      pspec = g_param_spec_uint ("mx-max-size",
                                  "Maximum grabber size",
                                  "Maximum size of the scroll grabber, in px",
                                  0, G_MAXINT16, G_MAXINT16,
@@ -606,7 +692,7 @@ move_slider (MxScrollBar *bar,
   if (!clutter_actor_transform_stage_point (priv->trough, x, y, &ux, &uy))
     return;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     size = clutter_actor_get_height (priv->trough)
            - clutter_actor_get_height (priv->handle);
   else
@@ -616,7 +702,7 @@ move_slider (MxScrollBar *bar,
   if (size == 0)
     return;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     pos = uy - priv->y_origin;
   else
     pos = ux - priv->x_origin;
@@ -715,14 +801,6 @@ handle_button_press_event_cb (ClutterActor       *actor,
   return TRUE;
 }
 
-static void
-animation_completed_cb (ClutterAnimation   *animation,
-                        MxScrollBarPrivate *priv)
-{
-  g_object_unref (priv->paging_animation);
-  priv->paging_animation = NULL;
-}
-
 static gboolean
 trough_paging_cb (MxScrollBar *self)
 {
@@ -730,11 +808,7 @@ trough_paging_cb (MxScrollBar *self)
   gdouble value;
   gdouble page_increment;
   gboolean ret;
-
   gulong mode;
-  ClutterAnimation *a;
-  GValue v = { 0, };
-  ClutterTimeline *t;
 
   if (self->priv->paging_event_no == 0)
     {
@@ -771,7 +845,7 @@ trough_paging_cb (MxScrollBar *self)
                             &value, NULL, NULL,
                             NULL, &page_increment, NULL);
 
-  if (self->priv->vertical)
+  if (self->priv->orientation == MX_ORIENTATION_VERTICAL)
     handle_pos = clutter_actor_get_y (self->priv->handle);
   else
     handle_pos = clutter_actor_get_x (self->priv->handle);
@@ -781,7 +855,7 @@ trough_paging_cb (MxScrollBar *self)
                                        self->priv->move_y,
                                        &tx, &ty);
 
-  if (self->priv->vertical)
+  if (self->priv->orientation == MX_ORIENTATION_VERTICAL)
     event_pos = ty;
   else
     event_pos = tx;
@@ -815,25 +889,7 @@ trough_paging_cb (MxScrollBar *self)
       value -= page_increment;
     }
 
-  if (self->priv->paging_animation)
-    {
-      clutter_animation_completed (self->priv->paging_animation);
-    }
-
-  /* FIXME: Creating a new animation for each scroll is probably not the best
-  * idea, but it's a lot less involved than extenind the current animation */
-  a = self->priv->paging_animation = g_object_new (CLUTTER_TYPE_ANIMATION,
-                                                   "object", self->priv->adjustment,
-                                                   "duration", PAGING_SUBSEQUENT_REPEAT_TIMEOUT,
-                                                   "mode", mode,
-                                                   NULL);
-  g_value_init (&v, G_TYPE_DOUBLE);
-  g_value_set_double (&v, value);
-  clutter_animation_bind (self->priv->paging_animation, "value", &v);
-  t = clutter_animation_get_timeline (self->priv->paging_animation);
-  g_signal_connect (a, "completed", G_CALLBACK (animation_completed_cb),
-                    self->priv);
-  clutter_timeline_start (t);
+  mx_adjustment_interpolate (self->priv->adjustment, value, 250, mode);
 
   return ret;
 }
@@ -892,31 +948,12 @@ trough_leave_event_cb (ClutterActor *actor,
   return FALSE;
 }
 
-static void
-stepper_animation_completed_cb (ClutterAnimation *a,
-                                gpointer          data)
-{
-  g_object_unref (a);
-}
-
 static void
 stepper_move_on (MxScrollBarPrivate *priv,
                  gint                mode)
 {
-  ClutterAnimation *a;
-  ClutterTimeline *t;
-  GValue v = { 0, };
   double value, inc;
 
-  a = g_object_new (CLUTTER_TYPE_ANIMATION,
-                    "object", priv->adjustment,
-                    "duration", PAGING_SUBSEQUENT_REPEAT_TIMEOUT,
-                    "mode", mode,
-                    NULL);
-
-  g_signal_connect (a, "completed", G_CALLBACK (stepper_animation_completed_cb),
-                    NULL);
-
   g_object_get (priv->adjustment,
                 "step-increment", &inc,
                 "value", &value,
@@ -927,12 +964,8 @@ stepper_move_on (MxScrollBarPrivate *priv,
   else
     value = value - inc;
 
-  g_value_init (&v, G_TYPE_DOUBLE);
-  g_value_set_double (&v, value);
-  clutter_animation_bind (a, "value", &v);
-
-  t = clutter_animation_get_timeline (a);
-  clutter_timeline_start (t);
+  mx_adjustment_interpolate (priv->adjustment, value,
+                             PAGING_SUBSEQUENT_REPEAT_TIMEOUT, mode);
 }
 
 static gboolean
@@ -1066,8 +1099,29 @@ mx_scroll_bar_init (MxScrollBar *self)
                     G_CALLBACK (mx_scroll_bar_notify_reactive), NULL);
 }
 
+/**
+ * mx_scroll_bar_new:
+ *
+ * Create a new #MxScrollBar
+ *
+ * Returns: a new #MxScrollBar
+ */
 ClutterActor *
-mx_scroll_bar_new (MxAdjustment *adjustment)
+mx_scroll_bar_new (void)
+{
+  return g_object_new (MX_TYPE_SCROLL_BAR, NULL);
+}
+
+/**
+ * mx_scroll_bar_new_with_adjustment:
+ * @adjustment: an #MxAdjustment
+ *
+ * Create a new #MxScrollBar with the given adjustment set
+ *
+ * Returns: a new #MxScrollBar
+ */
+ClutterActor *
+mx_scroll_bar_new_with_adjustment (MxAdjustment *adjustment)
 {
   return g_object_new (MX_TYPE_SCROLL_BAR,
                        "adjustment", adjustment,
@@ -1110,6 +1164,15 @@ mx_scroll_bar_set_adjustment (MxScrollBar  *bar,
     }
 }
 
+/**
+ * mx_scroll_bar_get_adjustment:
+ * @bar: a #MxScrollbar
+ *
+ * Gets the adjustment object that stores the current position
+ * of the scrollbar.
+ *
+ * Return value: (transfer none): the adjustment
+ */
 MxAdjustment *
 mx_scroll_bar_get_adjustment (MxScrollBar *bar)
 {
@@ -1118,3 +1181,51 @@ mx_scroll_bar_get_adjustment (MxScrollBar *bar)
   return bar->priv->adjustment;
 }
 
+
+MxOrientation
+mx_scroll_bar_get_orientation (MxScrollBar *bar)
+{
+  g_return_val_if_fail (MX_IS_SCROLL_BAR (bar), MX_ORIENTATION_HORIZONTAL);
+
+  return bar->priv->orientation;
+}
+
+void
+mx_scroll_bar_set_orientation (MxScrollBar   *bar,
+                               MxOrientation  orientation)
+{
+  MxScrollBarPrivate *priv;
+
+  g_return_if_fail (MX_IS_SCROLL_BAR (bar));
+
+  priv = bar->priv;
+
+  if (orientation != priv->orientation)
+    {
+
+      priv->orientation = orientation;
+
+      if (bar->priv->orientation)
+        {
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->bw_stepper),
+                                  "up-stepper");
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->fw_stepper),
+                                  "down-stepper");
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->handle),
+                                  "vhandle");
+        }
+      else
+        {
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->fw_stepper),
+                                  "forward-stepper");
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->bw_stepper),
+                                  "backward-stepper");
+          clutter_actor_set_name (CLUTTER_ACTOR (priv->handle),
+                                  "hhandle");
+        }
+
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (bar));
+
+      g_object_notify (G_OBJECT (bar), "orientation");
+    }
+}
