@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-table.c: Table layout widget
  *
@@ -61,14 +62,14 @@
 #include "mx-private.h"
 #include "mx-table-child.h"
 #include "mx-stylable.h"
+#include "mx-focusable.h"
 
 enum
 {
   PROP_0,
 
   PROP_PADDING,
-
-  PROP_COL_SPACING,
+  PROP_COLUMN_SPACING,
   PROP_ROW_SPACING,
 
   PROP_ROW_COUNT,
@@ -82,6 +83,7 @@ typedef struct
 {
   guint expand : 1;
   guint shrink : 1;
+  guint is_visible : 1;
 
   gfloat min_size;
   gfloat pref_size;
@@ -91,11 +93,14 @@ typedef struct
 
 struct _MxTablePrivate
 {
-  GSList *children;
+  GList *children;
 
   gint    col_spacing;
   gint    row_spacing;
 
+  gint    visible_rows;
+  gint    visible_cols;
+
   gint    n_rows;
   gint    n_cols;
 
@@ -104,15 +109,302 @@ struct _MxTablePrivate
 
   GArray *columns;
   GArray *rows;
+
+  MxFocusable *last_focus;
 };
 
 static void mx_container_iface_init (ClutterContainerIface *iface);
+static void mx_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxTable, mx_table, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
-                                                mx_container_iface_init));
+                                                mx_container_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_focusable_iface_init));
+
+static ClutterActor*
+mx_table_find_actor_at (MxTable *table,
+                        int      row,
+                        int      column)
+{
+  MxTablePrivate *priv;
+  GList *l;
+
+  priv = table->priv;
+
+  for (l = priv->children; l; l = g_list_next (l))
+    {
+      MxTableChild *child;
+      ClutterActor *actor_child = CLUTTER_ACTOR (l->data);
+
+      child = (MxTableChild *) clutter_container_get_child_meta (CLUTTER_CONTAINER (table),
+                                                                 actor_child);
+
+      if ((row >= child->row && row <= child->row + (child->row_span - 1)) &&
+          (column >= child->col && column <= child->col + (child->col_span - 1)))
+        return actor_child;
+    }
+
+  return NULL;
+}
+
+static MxFocusable*
+mx_table_move_focus (MxFocusable      *focusable,
+                     MxFocusDirection  direction,
+                     MxFocusable      *from)
+{
+  MxTablePrivate *priv = MX_TABLE (focusable)->priv;
+  MxTable *table = MX_TABLE (focusable);
+  GList *l, *childlink;
+  MxTableChild *child_meta;
+  ClutterActor *child_actor;
+  MxFocusable *focused;
+  gint row, column;
+  ClutterActor *found;
+
+  /* find the current focus */
+  childlink = g_list_find (priv->children, from);
+
+  if (!childlink)
+    return NULL;
+
+  child_actor = CLUTTER_ACTOR (childlink->data);
+  child_meta = (MxTableChild *) clutter_container_get_child_meta (CLUTTER_CONTAINER (focusable),
+                                                                  child_actor);
+
+  /* find the next widget to focus */
+  switch (direction)
+    {
+    case MX_FOCUS_DIRECTION_NEXT:
+
+      for (l = childlink->next; l; l = g_list_next (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                   MX_FOCUS_HINT_FIRST);
+
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)l->data;
+                  return focused;
+                }
+            }
+        }
+
+      /* no next widgets to focus */
+      return NULL;
+
+    case MX_FOCUS_DIRECTION_PREVIOUS:
+
+      for (l = g_list_previous (childlink); l; l = g_list_previous (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                   MX_FOCUS_HINT_LAST);
+
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)l->data;
+                  return focused;
+                }
+            }
+        }
+
+      /* no widget found in the previous position */
+      return NULL;
+
+    case MX_FOCUS_DIRECTION_UP:
+      /* move focus up */
+
+      row = child_meta->row - 1;
+      column = child_meta->col;
+
+      focused = NULL;
+
+      while (!focused && row >= 0)
+        {
+
+          found = mx_table_find_actor_at (table, row, column);
+
+          if (MX_IS_FOCUSABLE (found))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                   MX_FOCUS_HINT_FIRST);
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)found;
+                  break;
+                }
+            }
+
+          child_meta = (MxTableChild *) clutter_container_get_child_meta
+            (CLUTTER_CONTAINER (focusable), found);
+
+          /* row might not be the top row if @found is a spanned actor */
+          row = child_meta->row - 1;
+        }
+
+      return focused;
 
 
+    case MX_FOCUS_DIRECTION_DOWN:
+      /* move focus down */
+
+      row = child_meta->row + child_meta->row_span;
+      column = child_meta->col;
+
+      focused = NULL;
+
+      while (!focused && row < priv->n_rows)
+        {
+          found = mx_table_find_actor_at (table, row, column);
+
+          if (MX_IS_FOCUSABLE (found))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                   MX_FOCUS_HINT_FIRST);
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)found;
+                  break;
+                }
+            }
+
+          child_meta = (MxTableChild *) clutter_container_get_child_meta
+            (CLUTTER_CONTAINER (focusable), found);
+
+          row = child_meta->row + child_meta->row_span;
+        }
+
+      return focused;
+
+
+    case MX_FOCUS_DIRECTION_LEFT:
+      /* move focus left */
+
+      row = child_meta->row;
+      column = child_meta->col - 1;
+
+      focused = NULL;
+
+      while (!focused && column >= 0)
+        {
+          found = mx_table_find_actor_at (table, row, column);
+
+          if (MX_IS_FOCUSABLE (found))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                   MX_FOCUS_HINT_FIRST);
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)found;
+                  break;
+                }
+            }
+
+          child_meta = (MxTableChild *) clutter_container_get_child_meta
+            (CLUTTER_CONTAINER (focusable), found);
+
+          /* col might not be the first column if @found is a spanned actor */
+          column = child_meta->col - 1;
+        }
+
+      return focused;
+
+
+    case MX_FOCUS_DIRECTION_RIGHT:
+      /* move focus right */
+
+      row = child_meta->row;
+      column = child_meta->col + child_meta->col_span;
+
+      focused = NULL;
+
+      while (!focused && column < priv->n_cols)
+        {
+          found = mx_table_find_actor_at (table, row, column);
+
+          if (MX_IS_FOCUSABLE (found))
+            {
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                   MX_FOCUS_HINT_FIRST);
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)found;
+                  break;
+                }
+            }
+
+          child_meta = (MxTableChild *) clutter_container_get_child_meta
+            (CLUTTER_CONTAINER (focusable), found);
+
+          column = child_meta->col + child_meta->col_span;
+        }
+
+      return focused;
+
+    default:
+      break;
+    }
+
+  return NULL;
+}
+
+static MxFocusable*
+mx_table_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  MxTablePrivate *priv = MX_TABLE (focusable)->priv;
+  MxFocusable *return_focusable;
+  GList* list, *l;
+
+  return_focusable = NULL;
+
+  /* find the first/last focusable widget */
+  switch (hint)
+    {
+    case MX_FOCUS_HINT_LAST:
+      list = g_list_reverse (g_list_copy (priv->children));
+      break;
+
+    case MX_FOCUS_HINT_PRIOR:
+      if (priv->last_focus)
+        {
+          list = g_list_copy (g_list_find (priv->children, priv->last_focus));
+          if (list)
+            break;
+        }
+      /* This intentionally runs into the next switch case */
+
+    default:
+    case MX_FOCUS_HINT_FIRST:
+      list = g_list_copy (priv->children);
+      break;
+    }
+
+  for (l = list; l; l = g_list_next (l))
+    {
+      if (MX_IS_FOCUSABLE (l->data))
+        {
+          return_focusable = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                        hint);
+          if (return_focusable)
+            break;
+        }
+    }
+
+  g_list_free (list);
+
+  return return_focusable;
+}
+
+static void
+mx_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->move_focus = mx_table_move_focus;
+  iface->accept_focus = mx_table_accept_focus;
+}
 
 /*
  * ClutterContainer Implementation
@@ -126,7 +418,7 @@ mx_container_add_actor (ClutterContainer *container,
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
 
 
-  priv->children = g_slist_append (priv->children, actor);
+  priv->children = g_list_append (priv->children, actor);
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 
@@ -140,11 +432,11 @@ mx_container_remove_actor (ClutterContainer *container,
   MxTablePrivate *priv = MX_TABLE (container)->priv;
   gint rows, cols;
 
-  GSList *item = NULL;
+  GList *item = NULL;
   MxTableChild *meta;
-  GSList *l;
+  GList *l;
 
-  item = g_slist_find (priv->children, actor);
+  item = g_list_find (priv->children, actor);
 
   if (item == NULL)
     {
@@ -156,7 +448,10 @@ mx_container_remove_actor (ClutterContainer *container,
 
   g_object_ref (actor);
 
-  priv->children = g_slist_delete_link (priv->children, item);
+  if ((ClutterActor *)priv->last_focus == actor)
+    priv->last_focus = NULL;
+
+  priv->children = g_list_delete_link (priv->children, item);
   clutter_actor_unparent (actor);
 
   /* update row/column count */
@@ -167,8 +462,8 @@ mx_container_remove_actor (ClutterContainer *container,
       ClutterActor *child = CLUTTER_ACTOR (l->data);
       meta = (MxTableChild *) clutter_container_get_child_meta (container,
                                                                   child);
-      rows = MAX (rows, meta->row + 1);
-      cols = MAX (cols, meta->col + 1);
+      rows = MAX (rows, meta->row + meta->row_span);
+      cols = MAX (cols, meta->col + meta->col_span);
     }
   priv->n_rows = rows;
   priv->n_cols = cols;
@@ -187,7 +482,7 @@ mx_container_foreach (ClutterContainer *container,
 {
   MxTablePrivate *priv = MX_TABLE (container)->priv;
 
-  g_slist_foreach (priv->children, (GFunc) callback, callback_data);
+  g_list_foreach (priv->children, (GFunc) callback, callback_data);
 }
 
 static void
@@ -239,8 +534,8 @@ mx_table_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_COL_SPACING:
-      mx_table_set_col_spacing (table, g_value_get_int (value));
+    case PROP_COLUMN_SPACING:
+      mx_table_set_column_spacing (table, g_value_get_int (value));
       break;
 
     case PROP_ROW_SPACING:
@@ -263,7 +558,7 @@ mx_table_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_COL_SPACING:
+    case PROP_COLUMN_SPACING:
       g_value_set_int (value, priv->col_spacing);
       break;
 
@@ -415,7 +710,7 @@ mx_table_calculate_col_widths (MxTable *table,
   gint i;
   MxTablePrivate *priv = table->priv;
   DimensionData *columns;
-  GSList *l;
+  GList *l;
   MxPadding padding;
 
   g_array_set_size (priv->columns, 0);
@@ -428,8 +723,13 @@ mx_table_calculate_col_widths (MxTable *table,
 
   for_width -= (int)(padding.left + padding.right);
 
+  /* Reset all the visible attributes for the columns */
+  priv->visible_cols = 0;
+  for (i = 0; i < priv->n_cols; i++)
+    columns[i].is_visible = FALSE;
+
   /* STAGE ONE: calculate column widths for non-spanned children */
-  for (l = priv->children; l; l = g_slist_next (l))
+  for (l = priv->children; l; l = g_list_next (l))
     {
       MxTableChild *meta;
       ClutterActor *child;
@@ -438,6 +738,9 @@ mx_table_calculate_col_widths (MxTable *table,
 
       child = CLUTTER_ACTOR (l->data);
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       meta = (MxTableChild *)
         clutter_container_get_child_meta (CLUTTER_CONTAINER (table), child);
 
@@ -446,6 +749,13 @@ mx_table_calculate_col_widths (MxTable *table,
 
       col = &columns[meta->col];
 
+      /* If this child is visible, then its column is visible */
+      if (!col->is_visible)
+        {
+          col->is_visible = TRUE;
+          priv->visible_cols++;
+        }
+
       clutter_actor_get_preferred_width (child, -1, &c_min, &c_pref);
 
       col->min_size = MAX (col->min_size, c_min);
@@ -454,7 +764,7 @@ mx_table_calculate_col_widths (MxTable *table,
     }
 
   /* STAGE TWO: take spanning children into account */
-  for (l = priv->children; l; l = g_slist_next (l))
+  for (l = priv->children; l; l = g_list_next (l))
     {
       MxTableChild *meta;
       ClutterActor *child;
@@ -466,6 +776,9 @@ mx_table_calculate_col_widths (MxTable *table,
 
       child = CLUTTER_ACTOR (l->data);
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       meta = (MxTableChild *)
         clutter_container_get_child_meta (CLUTTER_CONTAINER (table), child);
 
@@ -492,6 +805,14 @@ mx_table_calculate_col_widths (MxTable *table,
             {
               n_expand++;
             }
+
+          /* If this child is visible, then the columns it spans
+             are also visible */
+          if (!col->is_visible)
+            {
+              col->is_visible = TRUE;
+              priv->visible_cols++;
+            }
         }
       min_width += priv->col_spacing * (meta->col_span - 1);
       pref_width += priv->col_spacing * (meta->col_span - 1);
@@ -674,7 +995,7 @@ mx_table_calculate_row_heights (MxTable *table,
                                 gint     for_height)
 {
   MxTablePrivate *priv = MX_TABLE (table)->priv;
-  GSList *l;
+  GList *l;
   gint i;
   DimensionData *rows, *columns;
   MxPadding padding;
@@ -690,9 +1011,13 @@ mx_table_calculate_row_heights (MxTable *table,
 
   columns = (DimensionData*) priv->columns->data;
 
+  /* Reset the visible rows */
+  priv->visible_rows = 0;
+  for (i = 0; i < priv->n_rows; i++)
+    rows[i].is_visible = FALSE;
 
   /* STAGE ONE: calculate row heights for non-spanned children */
-  for (l = priv->children; l; l = g_slist_next (l))
+  for (l = priv->children; l; l = g_list_next (l))
     {
       MxTableChild *meta;
       ClutterActor *child;
@@ -701,6 +1026,9 @@ mx_table_calculate_row_heights (MxTable *table,
 
       child = CLUTTER_ACTOR (l->data);
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       meta = (MxTableChild *)
         clutter_container_get_child_meta (CLUTTER_CONTAINER (table), child);
 
@@ -709,6 +1037,13 @@ mx_table_calculate_row_heights (MxTable *table,
 
       row = &rows[meta->row];
 
+      /* If this child is visible, then its row is visible */
+      if (!row->is_visible)
+        {
+          row->is_visible = TRUE;
+          priv->visible_rows++;
+        }
+
       clutter_actor_get_preferred_height (child, columns[meta->col].final_size,
                                           &c_min, &c_pref);
 
@@ -720,7 +1055,7 @@ mx_table_calculate_row_heights (MxTable *table,
 
 
   /* STAGE TWO: take spanning children into account */
-  for (l = priv->children; l; l = g_slist_next (l))
+  for (l = priv->children; l; l = g_list_next (l))
     {
       MxTableChild *meta;
       ClutterActor *child;
@@ -732,6 +1067,9 @@ mx_table_calculate_row_heights (MxTable *table,
 
       child = CLUTTER_ACTOR (l->data);
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       meta = (MxTableChild *)
         clutter_container_get_child_meta (CLUTTER_CONTAINER (table), child);
 
@@ -759,6 +1097,13 @@ mx_table_calculate_row_heights (MxTable *table,
             {
               n_expand++;
             }
+
+          /* If this actor is visible, then all the rows is spans are visible */
+          if (!rows[i].is_visible)
+            {
+              rows[i].is_visible = TRUE;
+              priv->visible_rows++;
+            }
         }
       min_height += priv->row_spacing * (meta->row_span - 1);
       pref_height += priv->row_spacing * (meta->row_span - 1);
@@ -850,8 +1195,8 @@ mx_table_calculate_row_heights (MxTable *table,
           if (rows[i].expand)
             n_expand++;
         }
-      pref_height += priv->col_spacing * (priv->n_rows - 1);
-      min_height += priv->col_spacing * (priv->n_rows - 1);
+      pref_height += priv->row_spacing * (priv->n_rows - 1);
+      min_height += priv->row_spacing * (priv->n_rows - 1);
 
       if (for_height <= min_height)
         {
@@ -959,7 +1304,7 @@ mx_table_preferred_allocate (ClutterActor          *self,
                              const ClutterActorBox *box,
                              gboolean               flags)
 {
-  GSList *list;
+  GList *list;
   gint row_spacing, col_spacing;
   gint i, table_width, table_height;
   MxTable *table;
@@ -988,8 +1333,7 @@ mx_table_preferred_allocate (ClutterActor          *self,
   rows = (DimensionData *) priv->rows->data;
   columns = (DimensionData *) priv->columns->data;
 
-
-  for (list = priv->children; list; list = g_slist_next (list))
+  for (list = priv->children; list; list = g_list_next (list))
     {
       gint row, col, row_span, col_span;
       gint col_width, row_height;
@@ -1017,7 +1361,6 @@ mx_table_preferred_allocate (ClutterActor          *self,
       x_fill = meta->x_fill;
       y_fill = meta->y_fill;
 
-
       /* initialise the width and height */
       col_width = columns[col].final_size;
       row_height = rows[row].final_size;
@@ -1055,16 +1398,26 @@ mx_table_preferred_allocate (ClutterActor          *self,
         }
 
       /* calculate child x */
-      child_x = (int) padding.left
-                + col_spacing * col;
+      child_x = (int) padding.left;
       for (i = 0; i < col; i++)
-        child_x += columns[i].final_size;
+        {
+          if (columns[i].is_visible)
+            {
+              child_x += columns[i].final_size;
+              child_x += col_spacing;
+            }
+        }
 
       /* calculate child y */
-      child_y = (int) padding.top
-                + row_spacing * row;
+      child_y = (int) padding.top;
       for (i = 0; i < row; i++)
-        child_y += rows[i].final_size;
+        {
+          if (rows[i].is_visible)
+            {
+              child_y += rows[i].final_size;
+              child_y += col_spacing;
+            }
+        }
 
       /* set up childbox */
       childbox.x1 = (float) child_x;
@@ -1123,7 +1476,7 @@ mx_table_get_preferred_width (ClutterActor *self,
   columns = (DimensionData*) priv->columns->data;
 
   total_min_width = padding.left + padding.right
-                    + (priv->n_cols - 1) * (float) priv->col_spacing;
+                    + (priv->visible_cols - 1) * (float) priv->col_spacing;
   total_pref_width = total_min_width;
 
   for (i = 0; i < priv->n_cols; i++)
@@ -1164,7 +1517,7 @@ mx_table_get_preferred_height (ClutterActor *self,
   mx_widget_get_padding (MX_WIDGET (self), &padding);
 
   /* start off with padding plus row spacing */
-  total_min_height = padding.top + padding.bottom + (priv->n_rows - 1) *
+  total_min_height = padding.top + padding.bottom + (priv->visible_rows - 1) *
                      (float)(priv->row_spacing);
 
   total_pref_height = total_min_height;
@@ -1185,17 +1538,52 @@ static void
 mx_table_paint (ClutterActor *self)
 {
   MxTablePrivate *priv = MX_TABLE (self)->priv;
-  GSList *list;
+  GList *list;
 
   /* make sure the background gets painted first */
   CLUTTER_ACTOR_CLASS (mx_table_parent_class)->paint (self);
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  for (list = priv->children; list; list = g_list_next (list))
     {
       ClutterActor *child = CLUTTER_ACTOR (list->data);
       if (CLUTTER_ACTOR_IS_VISIBLE (child))
         clutter_actor_paint (child);
     }
+
+  if (_mx_debug (MX_DEBUG_LAYOUT))
+    {
+      int i;
+      float width, height;
+      gfloat pos = 0;
+      DimensionData *rows, *cols;
+
+      rows = (DimensionData*) priv->rows->data;
+      cols = (DimensionData*) priv->columns->data;
+
+      clutter_actor_get_size (self, &width, &height);
+
+      cogl_set_source_color4f (0.0, 0.0, 1.0, 0.7);
+
+      for (i = 0; i < priv->n_rows; i++)
+        {
+          cogl_rectangle (0, pos, 10, pos + rows[i].final_size);
+
+          pos += rows[i].final_size + priv->row_spacing;
+        }
+
+
+      cogl_set_source_color4f (1.0, 0.0, 0.0, 0.7);
+
+      pos = 0;
+      for (i = 0; i < priv->n_rows; i++)
+        {
+          cogl_rectangle (pos, 0, pos + cols[i].final_size, 10);
+
+          pos += cols[i].final_size + priv->col_spacing;
+        }
+
+
+    }
 }
 
 static void
@@ -1203,12 +1591,12 @@ mx_table_pick (ClutterActor       *self,
                const ClutterColor *color)
 {
   MxTablePrivate *priv = MX_TABLE (self)->priv;
-  GSList *list;
+  GList *list;
 
   /* Chain up so we get a bounding box painted (if we are reactive) */
   CLUTTER_ACTOR_CLASS (mx_table_parent_class)->pick (self, color);
 
-  for (list = priv->children; list; list = g_slist_next (list))
+  for (list = priv->children; list; list = g_list_next (list))
     {
       if (CLUTTER_ACTOR_IS_VISIBLE (list->data))
         clutter_actor_paint (CLUTTER_ACTOR (list->data));
@@ -1219,7 +1607,7 @@ static void
 mx_table_show_all (ClutterActor *table)
 {
   MxTablePrivate *priv = MX_TABLE (table)->priv;
-  GSList *l;
+  GList *l;
 
   for (l = priv->children; l; l = l->next)
     clutter_actor_show_all (CLUTTER_ACTOR (l->data));
@@ -1231,7 +1619,7 @@ static void
 mx_table_hide_all (ClutterActor *table)
 {
   MxTablePrivate *priv = MX_TABLE (table)->priv;
-  GSList *l;
+  GList *l;
 
   clutter_actor_hide (table);
 
@@ -1263,13 +1651,14 @@ mx_table_class_init (MxTableClass *klass)
   actor_class->show_all = mx_table_show_all;
   actor_class->hide_all = mx_table_hide_all;
 
-  pspec = g_param_spec_int ("col-spacing",
+
+  pspec = g_param_spec_int ("column-spacing",
                             "Column Spacing",
                             "Spacing between columns",
                             0, G_MAXINT, 0,
                             MX_PARAM_READWRITE);
   g_object_class_install_property (gobject_class,
-                                   PROP_COL_SPACING,
+                                   PROP_COLUMN_SPACING,
                                    pspec);
 
   pspec = g_param_spec_int ("row-spacing",
@@ -1341,15 +1730,15 @@ mx_table_new (void)
 }
 
 /**
- * mx_table_set_col_spacing
+ * mx_table_set_column_spacing
  * @table: a #MxTable
  * @spacing: spacing in pixels
  *
  * Sets the amount of spacing between columns.
  */
 void
-mx_table_set_col_spacing (MxTable *table,
-                          gint     spacing)
+mx_table_set_column_spacing (MxTable *table,
+                             gint     spacing)
 {
   MxTablePrivate *priv;
 
@@ -1364,7 +1753,7 @@ mx_table_set_col_spacing (MxTable *table,
 
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 
-      g_object_notify (G_OBJECT (table), "col-spacing");
+      g_object_notify (G_OBJECT (table), "column-spacing");
     }
 }
 
@@ -1416,7 +1805,7 @@ mx_table_get_row_spacing (MxTable *table)
 }
 
 /**
- * mx_table_get_col_spacing
+ * mx_table_get_column_spacing
  * @table: a #MxTable
  *
  * Gets the amount of spacing between columns.
@@ -1424,7 +1813,7 @@ mx_table_get_row_spacing (MxTable *table)
  * Returns: the spacing between columns in device units
  */
 gint
-mx_table_get_col_spacing (MxTable *table)
+mx_table_get_column_spacing (MxTable *table)
 {
   MxTablePrivate *priv;
 
