@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-widget.c: Base class for Mx actors
  *
@@ -42,7 +43,7 @@
 #include "mx-tooltip.h"
 #include "mx-enum-types.h"
 
-#define LONG_PRESS_TIMOUT 500
+#define LONG_PRESS_TIMEOUT 500
 
 /*
  * Forward declaration for sake of MxWidgetChild
@@ -57,14 +58,15 @@ struct _MxWidgetPrivate
   gchar        *style_class;
 
   ClutterActor *border_image;
+  ClutterActor *old_border_image;
   ClutterActor *background_image;
   ClutterColor *bg_color;
 
-  gboolean      has_tooltip : 1;
-  gboolean      is_style_dirty : 1;
-  gboolean      is_hovered : 1;
+  guint         is_hovered : 1;
+  guint         is_disabled : 1;
 
   MxTooltip    *tooltip;
+  MxMenu       *menu;
 
   guint         long_press_source;
 };
@@ -88,8 +90,10 @@ enum
   PROP_STYLE_CLASS,
   PROP_STYLE_PSEUDO_CLASS,
 
-  PROP_HAS_TOOLTIP,
-  PROP_TOOLTIP_TEXT
+  PROP_TOOLTIP_TEXT,
+  PROP_MENU,
+
+  PROP_DISABLED
 };
 
 enum
@@ -133,14 +137,18 @@ mx_widget_set_property (GObject      *gobject,
       mx_stylable_set_style_class (MX_STYLABLE (actor), g_value_get_string (value));
       break;
 
-    case PROP_HAS_TOOLTIP:
-      mx_widget_set_has_tooltip (actor, g_value_get_boolean (value));
-      break;
-
     case PROP_TOOLTIP_TEXT:
       mx_widget_set_tooltip_text (actor, g_value_get_string (value));
       break;
 
+    case PROP_MENU:
+      mx_widget_set_menu (actor, (MxMenu *)g_value_get_object (value));
+      break;
+
+    case PROP_DISABLED:
+      mx_widget_set_disabled (actor, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -170,14 +178,18 @@ mx_widget_get_property (GObject    *gobject,
       g_value_set_string (value, priv->style_class);
       break;
 
-    case PROP_HAS_TOOLTIP:
-      g_value_set_boolean (value, priv->has_tooltip);
-      break;
-
     case PROP_TOOLTIP_TEXT:
       g_value_set_string (value, mx_widget_get_tooltip_text (actor));
       break;
 
+    case PROP_MENU:
+      g_value_set_object (value, mx_widget_get_menu (actor));
+      break;
+
+    case PROP_DISABLED:
+      g_value_set_boolean (value, mx_widget_get_disabled (actor));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -202,6 +214,14 @@ mx_widget_dispose (GObject *gobject)
       priv->border_image = NULL;
     }
 
+  if (priv->old_border_image)
+    {
+      g_object_remove_weak_pointer (G_OBJECT (priv->old_border_image),
+                                    (gpointer)&priv->old_border_image);
+      clutter_actor_unparent (priv->old_border_image);
+      priv->old_border_image = NULL;
+    }
+
   if (priv->background_image)
     {
       clutter_actor_unparent (priv->background_image);
@@ -214,6 +234,12 @@ mx_widget_dispose (GObject *gobject)
       priv->tooltip = NULL;
     }
 
+  if (priv->menu)
+    {
+      clutter_actor_unparent (CLUTTER_ACTOR (priv->menu));
+      priv->menu = NULL;
+    }
+
   G_OBJECT_CLASS (mx_widget_parent_class)->dispose (gobject);
 }
 
@@ -274,6 +300,9 @@ mx_widget_allocate (ClutterActor          *actor,
       clutter_actor_allocate (CLUTTER_ACTOR (priv->border_image),
                               &frame_box,
                               flags);
+
+      if (priv->old_border_image)
+        clutter_actor_allocate (priv->old_border_image, &frame_box, flags);
     }
 
   if (priv->background_image)
@@ -283,7 +312,9 @@ mx_widget_allocate (ClutterActor          *actor,
       };
       gfloat w, h;
 
-      clutter_actor_get_size (CLUTTER_ACTOR (priv->background_image), &w, &h);
+      clutter_actor_get_preferred_size (CLUTTER_ACTOR (priv->background_image),
+                                        NULL, NULL,
+                                        &w, &h);
 
       /* scale the background into the allocated bounds */
       if (w > frame_box.x2 || h > frame_box.y2)
@@ -335,6 +366,9 @@ mx_widget_allocate (ClutterActor          *actor,
   if (priv->tooltip)
     clutter_actor_allocate_preferred_size (CLUTTER_ACTOR (priv->tooltip),
                                            flags);
+  if (priv->menu)
+    clutter_actor_allocate_preferred_size (CLUTTER_ACTOR (priv->menu),
+                                           flags);
 }
 
 static void
@@ -370,6 +404,9 @@ mx_widget_real_paint_background (MxWidget           *self,
 
   if (background)
     clutter_actor_paint (background);
+
+  if (self->priv->old_border_image)
+    clutter_actor_paint (self->priv->old_border_image);
 }
 
 static void
@@ -387,24 +424,20 @@ mx_widget_paint (ClutterActor *self)
 
   if (priv->tooltip)
     clutter_actor_paint (CLUTTER_ACTOR (priv->tooltip));
+
+  if (priv->menu)
+    clutter_actor_paint (CLUTTER_ACTOR (priv->menu));
 }
 
 static void
-mx_widget_parent_set (ClutterActor *widget,
-                      ClutterActor *old_parent)
+mx_widget_pick (ClutterActor *self, const ClutterColor *color)
 {
-  ClutterActorClass *parent_class;
-  ClutterActor *new_parent;
+  MxWidgetPrivate *priv = MX_WIDGET (self)->priv;
 
-  parent_class = CLUTTER_ACTOR_CLASS (mx_widget_parent_class);
-  if (parent_class->parent_set)
-    parent_class->parent_set (widget, old_parent);
+  CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->pick (self, color);
 
-  new_parent = clutter_actor_get_parent (widget);
-
-  /* don't send the style changed signal if we no longer have a parent actor */
-  if (new_parent)
-    mx_stylable_changed ((MxStylable*) widget);
+  if (priv->menu)
+    clutter_actor_paint (CLUTTER_ACTOR (priv->menu));
 }
 
 static void
@@ -414,16 +447,20 @@ mx_widget_map (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->map (actor);
 
-  mx_widget_ensure_style ((MxWidget*) actor);
-
   if (priv->border_image)
     clutter_actor_map (priv->border_image);
 
+  if (priv->old_border_image)
+    clutter_actor_map (priv->old_border_image);
+
   if (priv->background_image)
     clutter_actor_map (priv->background_image);
 
   if (priv->tooltip)
     clutter_actor_map (CLUTTER_ACTOR (priv->tooltip));
+
+  if (priv->menu)
+    clutter_actor_map (CLUTTER_ACTOR (priv->menu));
 }
 
 static void
@@ -436,15 +473,27 @@ mx_widget_unmap (ClutterActor *actor)
   if (priv->border_image)
     clutter_actor_unmap (priv->border_image);
 
+  if (priv->old_border_image)
+    clutter_actor_unmap (CLUTTER_ACTOR (priv->old_border_image));
+
   if (priv->background_image)
     clutter_actor_unmap (priv->background_image);
 
   if (priv->tooltip)
     clutter_actor_unmap (CLUTTER_ACTOR (priv->tooltip));
+
+  if (priv->menu)
+    clutter_actor_unmap (CLUTTER_ACTOR (priv->menu));
 }
 
 static void
-mx_widget_style_changed (MxStylable *self)
+old_background_faded_cb (ClutterAnimation *animation, ClutterActor *self)
+{
+  clutter_actor_unparent (self);
+}
+
+static void
+mx_widget_style_changed (MxStylable *self, MxStyleChangedFlags flags)
 {
   MxWidgetPrivate *priv = MX_WIDGET (self)->priv;
   MxBorderImage *border_image = NULL, *background_image = NULL;
@@ -455,6 +504,7 @@ mx_widget_style_changed (MxStylable *self)
   gboolean relayout_needed = FALSE;
   gboolean has_changed = FALSE;
   ClutterColor *color;
+  guint duration;
 
   /* cache these values for use in the paint function */
   mx_stylable_get (self,
@@ -462,6 +512,7 @@ mx_widget_style_changed (MxStylable *self)
                    "background-image", &background_image,
                    "border-image", &border_image,
                    "padding", &padding,
+                   "x-mx-border-image-transition-duration", &duration,
                    NULL);
 
   if (color)
@@ -506,7 +557,32 @@ mx_widget_style_changed (MxStylable *self)
 
   if (priv->border_image)
     {
-      clutter_actor_unparent (priv->border_image);
+      if (duration == 0)
+        {
+          clutter_actor_unparent (priv->border_image);
+        }
+      else
+        {
+          if (priv->old_border_image)
+            {
+              g_object_remove_weak_pointer (G_OBJECT (priv->old_border_image),
+                                            (gpointer)&priv->old_border_image);
+              clutter_actor_unparent (priv->old_border_image);
+            }
+
+          priv->old_border_image = priv->border_image;
+          g_object_add_weak_pointer (G_OBJECT (priv->old_border_image),
+                                     (gpointer)&priv->old_border_image);
+          clutter_actor_animate (priv->old_border_image,
+                                 CLUTTER_LINEAR,
+                                 duration,
+                                 "opacity", 0x0,
+                                 "signal-after::completed",
+                                   old_background_faded_cb,
+                                   priv->old_border_image,
+                                 NULL);
+        }
+
       priv->border_image = NULL;
     }
 
@@ -588,38 +664,6 @@ mx_widget_style_changed (MxStylable *self)
       else
         clutter_actor_queue_redraw ((ClutterActor *) self);
     }
-
-  priv->is_style_dirty = FALSE;
-}
-
-static void
-mx_widget_stylable_child_notify (ClutterActor *actor,
-                                 gpointer      user_data)
-{
-  if (MX_IS_STYLABLE (actor))
-    mx_stylable_changed ((MxStylable*) actor);
-}
-
-static void
-mx_widget_stylable_changed (MxStylable *stylable)
-{
-
-  MX_WIDGET (stylable)->priv->is_style_dirty = TRUE;
-
-  /* update the style only if we are mapped */
-  if (!CLUTTER_ACTOR_IS_MAPPED ((ClutterActor *) stylable))
-    return;
-
-  g_signal_emit_by_name (stylable, "style-changed", 0);
-
-
-  if (CLUTTER_IS_CONTAINER (stylable))
-    {
-      /* notify our children that their parent stylable has changed */
-      clutter_container_foreach ((ClutterContainer *) stylable,
-                                 mx_widget_stylable_child_notify,
-                                 NULL);
-    }
 }
 
 static gboolean
@@ -632,12 +676,15 @@ mx_widget_enter (ClutterActor         *actor,
   if (event->source != actor)
     return FALSE;
 
+  if (priv->tooltip)
+    mx_widget_show_tooltip (widget);
+
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+      return TRUE;
+
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
   priv->is_hovered = TRUE;
 
-  if (priv->has_tooltip)
-    mx_widget_show_tooltip (widget);
-
   return FALSE;
 }
 
@@ -651,12 +698,15 @@ mx_widget_leave (ClutterActor         *actor,
   if (event->source != actor)
     return FALSE;
 
+  if (priv->tooltip)
+    mx_tooltip_hide (priv->tooltip);
+
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+      return TRUE;
+
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
   priv->is_hovered = FALSE;
 
-  if (priv->has_tooltip)
-    mx_tooltip_hide (priv->tooltip);
-
   return FALSE;
 }
 
@@ -690,7 +740,7 @@ mx_widget_long_press_query (MxWidget           *widget,
                  event->y, MX_LONG_PRESS_QUERY, &query_result);
 
   if (query_result)
-    priv->long_press_source = g_timeout_add (LONG_PRESS_TIMOUT,
+    priv->long_press_source = g_timeout_add (LONG_PRESS_TIMEOUT,
                                              (GSourceFunc) mx_widget_emit_long_press,
                                              widget);
 }
@@ -724,6 +774,9 @@ mx_widget_button_press (ClutterActor       *actor,
 {
   MxWidget *widget = MX_WIDGET (actor);
 
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+      return TRUE;
+
   if (event->button == 1)
     mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "active");
 
@@ -738,6 +791,9 @@ mx_widget_button_release (ClutterActor       *actor,
 {
   MxWidget *widget = MX_WIDGET (actor);
 
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+      return TRUE;
+
   if (event->button == 1)
     {
       if (widget->priv->is_hovered)
@@ -781,7 +837,7 @@ mx_widget_class_init (MxWidgetClass *klass)
 
   actor_class->allocate = mx_widget_allocate;
   actor_class->paint = mx_widget_paint;
-  actor_class->parent_set = mx_widget_parent_set;
+  actor_class->pick = mx_widget_pick;
   actor_class->map = mx_widget_map;
   actor_class->unmap = mx_widget_unmap;
 
@@ -801,22 +857,6 @@ mx_widget_class_init (MxWidgetClass *klass)
   g_object_class_override_property (gobject_class, PROP_STYLE_PSEUDO_CLASS,
                                     "style-pseudo-class");
 
-  /**
-   * MxWidget:has-tooltip:
-   *
-   * Determines whether the widget has a tooltip. If set to TRUE, causes the
-   * widget to monitor enter and leave events (i.e. sets the widget reactive).
-   */
-  pspec = g_param_spec_boolean ("has-tooltip",
-                                "Has Tooltip",
-                                "Determines whether the widget has a tooltip",
-                                FALSE,
-                                MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_HAS_TOOLTIP,
-                                   pspec);
-
-
   /**
    * MxWidget:tooltip-text:
    *
@@ -829,6 +869,28 @@ mx_widget_class_init (MxWidgetClass *klass)
                                MX_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT, pspec);
 
+  /**
+   * MxWidget:menu:
+   *
+   * #MxMenu associated with the widget.
+   * See mx_widget_show_menu().
+   */
+  pspec = g_param_spec_object ("menu",
+                               "Menu",
+                               "The MxMenu associated with the widget",
+                               MX_TYPE_MENU,
+                               MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_MENU, pspec);
+
+  pspec = g_param_spec_boolean ("disabled",
+                               "Disabled",
+                               "Whether disabled styling should be applied and"
+                               " the widget made unreactive.",
+                               FALSE,
+                               MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_DISABLED, pspec);
+
+
 
   /**
    * MxWidget::long-press:
@@ -857,14 +919,6 @@ mx_widget_get_style (MxStylable *stylable)
   return priv->style;
 }
 
-static void
-mx_style_changed_cb (MxStyle    *style,
-                     MxStylable *stylable)
-{
-  mx_stylable_changed (stylable);
-}
-
-
 static void
 mx_widget_set_style (MxStylable *stylable,
                      MxStyle    *style)
@@ -875,11 +929,6 @@ mx_widget_set_style (MxStylable *stylable,
     g_object_unref (priv->style);
 
   priv->style = g_object_ref_sink (style);
-
-  g_signal_connect (priv->style,
-                    "changed",
-                    G_CALLBACK (mx_style_changed_cb),
-                    stylable);
 }
 
 static void
@@ -897,8 +946,6 @@ _mx_widget_set_style_class (MxStylable  *actor,
       g_free (priv->style_class);
       priv->style_class = g_strdup (style_class);
 
-      mx_stylable_changed (actor);
-
       g_object_notify (G_OBJECT (actor), "style-class");
     }
 }
@@ -918,8 +965,6 @@ _mx_stylable_set_style_pseudo_class (MxStylable  *actor,
       g_free (priv->pseudo_class);
       priv->pseudo_class = g_strdup (pseudo_class);
 
-      mx_stylable_changed (actor);
-
       g_object_notify (G_OBJECT (actor), "style-pseudo-class");
     }
 }
@@ -927,7 +972,10 @@ _mx_stylable_set_style_pseudo_class (MxStylable  *actor,
 static const gchar*
 _mx_stylable_get_style_pseudo_class (MxStylable *actor)
 {
-  return ((MxWidget *) actor)->priv->pseudo_class;
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return "disabled";
+  else
+    return ((MxWidget *) actor)->priv->pseudo_class;
 }
 
 
@@ -990,7 +1038,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
                                  "Font Weight",
                                  "Font Weight",
                                  MX_TYPE_FONT_WEIGHT,
-                                 MX_WEIGHT_NORMAL,
+                                 MX_FONT_WEIGHT_NORMAL,
                                  G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_WIDGET, pspec);
 
@@ -1009,8 +1057,31 @@ mx_stylable_iface_init (MxStylableIface *iface)
                                   G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_WIDGET, pspec);
 
+      pspec = g_param_spec_uint ("x-mx-border-image-transition-duration",
+                                 "background transition duration",
+                                 "Length of the cross fade when changing images"
+                                 " in milliseconds",
+                                 0, G_MAXUINT, 0,
+                                 G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_WIDGET, pspec);
+
+      /*
+      pspec = g_param_spec_uint ("x-mx-transition-duration",
+                                 "transition duration",
+                                 "Length of transition in milliseconds",
+                                 0, G_MAXUINT, 0,
+                                 G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_WIDGET, pspec);
+
+      pspec = g_param_spec_string ("x-mx-transition-property",
+                                   "transition property",
+                                   "Property to apply the transition too",
+                                   "",
+                                   G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_WIDGET, pspec);
+      */
+
       iface->style_changed = mx_widget_style_changed;
-      iface->stylable_changed = mx_widget_stylable_changed;
 
       iface->get_style = mx_widget_get_style;
       iface->set_style = mx_widget_set_style;
@@ -1022,15 +1093,6 @@ mx_stylable_iface_init (MxStylableIface *iface)
     }
 }
 
-
-static void
-mx_widget_name_notify (MxWidget   *widget,
-                       GParamSpec *pspec,
-                       gpointer    data)
-{
-  mx_stylable_changed ((MxStylable*) widget);
-}
-
 static void
 mx_widget_init (MxWidget *actor)
 {
@@ -1038,30 +1100,11 @@ mx_widget_init (MxWidget *actor)
 
   actor->priv = priv = MX_WIDGET_GET_PRIVATE (actor);
 
-  /* connect style changed */
-  g_signal_connect (actor, "notify::name", G_CALLBACK (mx_widget_name_notify), NULL);
-
   /* set the default style */
   mx_widget_set_style (MX_STYLABLE (actor), mx_style_get_default ());
 
-}
-
-/**
- * mx_widget_ensure_style:
- * @widget: A #MxWidget
- *
- * Ensures that @widget has read its style information.
- *
- */
-void
-mx_widget_ensure_style (MxWidget *widget)
-{
-  g_return_if_fail (MX_IS_WIDGET (widget));
-
-  if (widget->priv->is_style_dirty)
-    {
-      g_signal_emit_by_name (widget, "style-changed", 0);
-    }
+  /* connect the notifiers for the stylable */
+  mx_stylable_connect_change_notifiers (MX_STYLABLE (actor));
 }
 
 
@@ -1073,7 +1116,7 @@ mx_widget_ensure_style (MxWidget *widget)
  * "border-image" CSS property. This function should normally only be used
  * by subclasses.
  *
- * Returns: #ClutterActor
+ * Returns: (transfer none): #ClutterActor
  */
 ClutterActor *
 mx_widget_get_border_image (MxWidget *actor)
@@ -1090,7 +1133,7 @@ mx_widget_get_border_image (MxWidget *actor)
  * "background-image" CSS property. This function should normally only be used
  * by subclasses.
  *
- * Returns: a #ClutterActor
+ * Returns: (transfer none): a #ClutterActor
  */
 ClutterActor *
 mx_widget_get_background_image (MxWidget *actor)
@@ -1118,19 +1161,7 @@ mx_widget_get_padding (MxWidget  *widget,
   *padding = widget->priv->padding;
 }
 
-/**
- * mx_widget_set_has_tooltip:
- * @widget: A #MxWidget
- * @has_tooltip: #TRUE if the widget should display a tooltip
- *
- * Enables tooltip support on the #MxWidget.
- *
- * Note that setting has-tooltip to #TRUE will cause the widget to be set
- * reactive. If you no longer need tooltip support and do not need the widget
- * to be reactive, you need to set ClutterActor::reactive to FALSE.
- *
- */
-void
+static void
 mx_widget_set_has_tooltip (MxWidget *widget,
                            gboolean  has_tooltip)
 {
@@ -1143,8 +1174,6 @@ mx_widget_set_has_tooltip (MxWidget *widget,
 
   priv = widget->priv;
 
-  priv->has_tooltip = has_tooltip;
-
   if (has_tooltip)
     {
       clutter_actor_set_reactive (actor, TRUE);
@@ -1165,30 +1194,15 @@ mx_widget_set_has_tooltip (MxWidget *widget,
     }
 }
 
-/**
- * mx_widget_get_has_tooltip:
- * @widget: A #MxWidget
- *
- * Returns the current value of the has-tooltip property. See
- * mx_tooltip_set_has_tooltip() for more information.
- *
- * Returns: current value of has-tooltip on @widget
- */
-gboolean
-mx_widget_get_has_tooltip (MxWidget *widget)
-{
-  g_return_val_if_fail (MX_IS_WIDGET (widget), FALSE);
-
-  return widget->priv->has_tooltip;
-}
-
 /**
  * mx_widget_set_tooltip_text:
  * @widget: A #MxWidget
  * @text: text to set as the tooltip
  *
- * Set the tooltip text of the widget. This will set MxWidget::has-tooltip to
- * #TRUE. A value of #NULL will unset the tooltip and set has-tooltip to #FALSE.
+ * Set the tooltip text of the widget. Note that setting tooltip text will cause
+ * the widget to be set reactive. If you no longer need tooltips and you do not
+ * need the widget to be reactive, you must set ClutterActor::reactive to
+ * %FALSE.
  *
  */
 void
@@ -1196,17 +1210,31 @@ mx_widget_set_tooltip_text (MxWidget    *widget,
                             const gchar *text)
 {
   MxWidgetPrivate *priv;
+  const gchar *old_text;
 
   g_return_if_fail (MX_IS_WIDGET (widget));
 
   priv = widget->priv;
 
+  if (priv->tooltip)
+    old_text = mx_tooltip_get_text (priv->tooltip);
+  else
+    old_text = NULL;
+
+  /* Don't do anything if the text hasn't changed */
+  if ((text == old_text) ||
+      (text && old_text && g_str_equal (text, old_text)))
+    return;
+
   if (text == NULL)
     mx_widget_set_has_tooltip (widget, FALSE);
   else
     mx_widget_set_has_tooltip (widget, TRUE);
 
-  mx_tooltip_set_label (priv->tooltip, text);
+  if (priv->tooltip)
+    mx_tooltip_set_text (priv->tooltip, text);
+
+  g_object_notify (G_OBJECT (widget), "tooltip-text");
 }
 
 /**
@@ -1225,10 +1253,10 @@ mx_widget_get_tooltip_text (MxWidget *widget)
   g_return_val_if_fail (MX_IS_WIDGET (widget), NULL);
   priv = widget->priv;
 
-  if (!priv->has_tooltip)
+  if (!priv->tooltip)
     return NULL;
 
-  return mx_tooltip_get_label (widget->priv->tooltip);
+  return mx_tooltip_get_text (widget->priv->tooltip);
 }
 
 /**
@@ -1309,6 +1337,17 @@ mx_widget_paint_background (MxWidget *self)
 
 }
 
+/**
+ * mx_widget_get_available_area:
+ * @widget: A #MxWidget
+ * @allocation: A #ClutterActorBox
+ * @area: A #ClutterActorBox
+ *
+ * Copies @allocation into @area and accounts for the padding values. This
+ * gives the area that is available in which to allocate children with respect
+ * to padding.
+ *
+ */
 void
 mx_widget_get_available_area (MxWidget              *widget,
                               const ClutterActorBox *allocation,
@@ -1322,3 +1361,109 @@ mx_widget_get_available_area (MxWidget              *widget,
   area->x2 = allocation->x2 - allocation->x1 - priv->padding.right;
   area->y2 = allocation->y2 - allocation->y1 - priv->padding.bottom;
 }
+
+/**
+ * mx_widget_set_menu:
+ * @widget: A #MxWidget
+ * @menu: A #MxMenu
+ *
+ * Set the value of the #MxWidget:menu property.
+ *
+ */
+void
+mx_widget_set_menu (MxWidget *widget,
+                    MxMenu   *menu)
+{
+  MxWidgetPrivate *priv = widget->priv;
+
+  if (priv->menu)
+    {
+      clutter_actor_unparent (CLUTTER_ACTOR (priv->menu));
+      priv->menu = NULL;
+    }
+
+  if (menu)
+    {
+      priv->menu = menu;
+      clutter_actor_set_parent (CLUTTER_ACTOR (menu), CLUTTER_ACTOR (widget));
+    }
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (widget));
+}
+
+/**
+ * mx_widget_get_menu:
+ * @widget: A #MxWidget
+ *
+ * Get the object in the #MxWidget:menu property.
+ *
+ * Returns: The current object in the "menu" property.
+ */
+MxMenu *
+mx_widget_get_menu (MxWidget *widget)
+{
+  return widget->priv->menu;
+}
+
+/**
+ * mx_widget_set_disabled:
+ * @widget: an #MxWidget
+ * @disabled: value to set
+ *
+ * Set the disabled property. Disabled widgets have a "disabled" pseudo-class
+ * until disabled is set to #FALSE.
+ */
+void
+mx_widget_set_disabled (MxWidget *widget,
+                        gboolean  disabled)
+{
+  MxWidgetPrivate *priv;
+
+  g_return_if_fail (MX_IS_WIDGET (widget));
+
+  priv = widget->priv;
+
+  if (priv->is_disabled != disabled)
+    {
+      priv->is_disabled = disabled;
+
+      /* when a widget is disabled, get_style_pseudo_class will always return
+       * "disabled" */
+
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (widget));
+
+      mx_stylable_style_changed (MX_STYLABLE (widget), 0);
+
+      g_object_notify (G_OBJECT (widget), "disabled");
+    }
+}
+
+/**
+ * mx_widget_get_disabled:
+ * @widget: an #MxWidget
+ *
+ * Get the value of the "disabled" property.
+ */
+gboolean
+mx_widget_get_disabled (MxWidget *widget)
+{
+  ClutterActor *parent;
+
+  g_return_val_if_fail (MX_IS_WIDGET (widget), FALSE);
+
+  if (widget->priv->is_disabled)
+    return TRUE;
+
+  /* check if any of the parents are disabled */
+  parent = clutter_actor_get_parent (CLUTTER_ACTOR (widget));
+
+  while (parent)
+    {
+      if (MX_IS_WIDGET (parent) && MX_WIDGET (parent)->priv->is_disabled)
+        return TRUE;
+
+      parent = clutter_actor_get_parent (CLUTTER_ACTOR (parent));
+    }
+
+  return FALSE;
+}
